﻿using System;
using System.Text;

namespace PH.Bootloader.RS485.DSP.IntelHex
{
    /// <summary>封裝 Intel Hex 檔資料內容之類別。</summary>
    public class IntelHexRecord
    {
        /// <summary>取得、設定屬性。</summary>
        public IntelHexRecordType   Type { get; set; }
        /// <summary>取得、設定資料長度。</summary>
        public int                  DataLength { get; set; }
        /// <summary>取得、設定記憶體位置。</summary>
        public long                 Address { get; set; }
        /// <summary>取得、設定資料內容。</summary>
        public byte[]               Data { get; set; }
        /// <summary>取得Check Sum值。</summary>
        public byte                 Checksum { get { return GetCheckSum(); } }

        /// <summary>將物件內容轉成字串。</summary>
        /// <returns></returns>
        public override string ToString()
        {
            StringBuilder outcome = new StringBuilder();

            // 儲存採用『little endian』，顯示採用『big endian』
            byte[] addressBytes = BitConverter.GetBytes(Address);
            if (!BitConverter.IsLittleEndian) {
                Array.Reverse(addressBytes);
            }

            outcome.Append(string.Format("{0:X2}{1:X2}{2:X2}{3:X2}  ", addressBytes[3], addressBytes[2], addressBytes[1], addressBytes[0]));
            outcome.Append(string.Format("{0:X2} ", (byte)Type));
            for (int i = 0; i < DataLength; i++) {
                outcome.Append(string.Format("{0:X2} ", Data[i]));
            }
            outcome.Append(string.Format("{0:X2}", Checksum));

            return outcome.ToString();
        }

        /// <summary>複製物件內容。</summary>
        /// <param name="src"></param>
        public void Copy(IntelHexRecord src)
        {
            Type = src.Type;
            DataLength = src.DataLength;
            Address = src.Address;
            Data = new byte[src.Data.Length];
            Array.Copy(src.Data, 0, Data, 0, src.Data.Length);
        }

        /// <summary>取得 GetCheckSum 值。</summary>
        /// <returns></returns>
        private byte GetCheckSum()
        {
            byte checksum;

            checksum = (byte)DataLength;
            checksum += (byte)Type;
            checksum += (byte)Address;
            checksum += (byte)((Address & 0xFF00) >> 8);

            if (Data != null) {
                for (int i = 0; i < DataLength; i++) {
                    checksum += Data[i];
                }
            }

            checksum = (byte)(~checksum + 1);
            return checksum;
        }
    }
}
