﻿using System.Collections.Generic;

namespace PH.Bootloader.RS485
{
    /// <summary>列舉所有裝置型態。</summary>
    public class DeviceType
    {
        /// <summary>CSU (PIC)。</summary>
        public static DeviceType PIC_CSU = new DeviceType(0, "PIC (CSU)", "CSU");
        /// <summary>PSU (PIC)。</summary>
        public static DeviceType PIC_PSU = new DeviceType(1, "PIC (PSU)", "PSU");
        /// <summary>PFC (PIC)。</summary>
        public static DeviceType PIC_PFC = new DeviceType(2, "PIC (PFC)", "PSU");
        /// <summary>VCU (PIC)。</summary>
        public static DeviceType PIC_VCU = new DeviceType(3, "PIC (VCU)", "VCU");
        /// <summary>SBox (PIC)。</summary>
        public static DeviceType PIC_SBox = new DeviceType(4, "PIC (SBox)", "SBo");
        /// <summary>PSU (DSP)。</summary>
        public static DeviceType DSP_PSU = new DeviceType(5, "DSP (PSU)", "");
        /// <summary>PFC (DSP)。</summary>
        public static DeviceType DSP_PFC = new DeviceType(6, "DSP (PFC)", "");

        #region Properties
        /// <summary>取得索引指標。</summary>
        public int      Index { get; private set; }
        /// <summary>取得文字描述。</summary>
        public string   Name { get; private set; }
        /// <summary>取得 Hex 辨識碼。</summary>
        public string   HexKey { get; private set; }
        #endregion

        #region Constructor
        /// <summary>初始化 DeviceType 類別的新執行個體。</summary>
        /// <param name="index"></param>
        /// <param name="name"></param>
        /// <param name="hexKey"></param>
        public DeviceType(int index, string name, string hexKey)
        {
            Index = index;
            Name = name;
            HexKey = hexKey;
        }
        #endregion

        #region Methods
        /// <summary>比較兩物件。</summary>
        /// <param name="target"></param>
        /// <returns></returns>
        public bool Equals(DeviceType target)
        {
            return Index == target.Index;
        }

        /// <summary>取得所有裝置定義。</summary>
        /// <returns></returns>
        public static IEnumerable<DeviceType> List
        {
            get
            {
                yield return PIC_CSU;
                yield return PIC_PSU;
                yield return PIC_PFC;
                yield return PIC_VCU;
                yield return PIC_SBox;
                yield return DSP_PSU;
                yield return DSP_PFC;
            }
        }

        /// <summary>取得所有裝置定義名稱。</summary>
        /// <returns></returns>
        public static IEnumerable<string> ListDeviceName
        {
            get
            {
                yield return PIC_CSU.Name;
                yield return PIC_PSU.Name;
                yield return PIC_PFC.Name;
                yield return PIC_VCU.Name;
                yield return PIC_SBox.Name;
                yield return DSP_PSU.Name;
                yield return DSP_PFC.Name;
            }
        }

        /// <summary>根據索引值，解析出 PSU 通訊位址(站號)定義物件。</summary>
        /// <param name="rs485ID"></param>
        /// <returns></returns>
        public static DeviceType ParseByIndex(int index)
        {
            foreach (DeviceType device in List)
            {
                if (device.Index == index) {
                    return device;
                }
            }

            return null;
        }

        /// <summary>根據名稱，解析出 PSU 通訊位址(站號)定義物件。</summary>
        /// <param name="rs485ID"></param>
        /// <returns></returns>
        public static DeviceType ParseByName(string name)
        {
            foreach (DeviceType device in List)
            {
                if (device.Name == name) {
                    return device;
                }
            }

            return null;
        }
        #endregion
    }
}
