﻿using Net.Core.Communication.Serial.Sync;

namespace PH.Bootloader.RS485
{
    /// <summary>解析出 Hex 檔案內容的事件處理方法。</summary>
    /// <param name="sender"></param>
    /// <param name="records"></param>
    public delegate void HexParsedEventHandler(object sender, string[] records);
    /// <summary>更新程序信息顯示的事件處理方法。</summary>
    /// <param name="message"></param>
    public delegate void MessageShownEventHandler(object sender, string message);
    /// <summary>程序進度更新的事件處理方法。</summary>
    /// <param name="sender"></param>
    /// <param name="taskType"></param>
    /// <param name="progressPercentage"></param>
    public delegate void ProcedureProgressChangedEventHandler(object sender, ProcedureType procType, int progressPercentage);
    /// <summary>資料更新的事件處理方法。</summary>
    /// <param name="sender"></param>
    /// <param name="current"></param>
    /// <param name="total"></param>
    public delegate void UploadInformationChangedEventHandler(object sender, long current, long total);
    /// <summary>程序執行完成的事件處理方法。</summary>
    /// <param name="sender"></param>
    /// <param name="procType"></param>
    /// <param name="result"></param>
    public delegate void ProcedureCompletedEventHandler(object sender, ProcedureType procType, BootloadResult result);

    /// <summary>基於 RS-485 通訊界面的啟動程序安裝介面。</summary>
    public interface IBootloader
    {
        /// <summary>取得啟動程序名稱。</summary>
        string  Name { get; }
        /// <summary>取得程序執行中旗標。</summary>
        bool    IsProcedureRunning { get; }
        /// <summary>取得是否由 PIC 擔任代理燒錄的腳色 (透過 CAN Bus)(僅有 DSP 有該功能)。</summary>
        bool    IsCanAgent { get; set; }

        /// <summary>取消所有正在執行的程序。</summary>
        void Abort();

        /// <summary>指定串列通訊埠。</summary>
        /// <param name="comPort"></param>
        /// <returns></returns>
        bool AssignComPort(ISyncCommunicationPlus comPort);

        /// <summary>檢查裝置。</summary>
        /// <param name="device"></param>
        bool CheckDevice(DeviceInformation device);

        /// <summary>檢查 App Code 是否存在。</summary>
        /// <param name="device"></param>
        /// <returns></returns>
        bool CheckAppCode(DeviceInformation device);

        /// <summary>準備記憶體內容。</summary>
        /// <param name="device"></param>
        /// <param name="filePath"></param>
        void ParseHexToMemory(DeviceInformation device, string filePath);

        /// <summary>上載。</summary>
        /// <param name="device"></param>
        /// <returns></returns>
        bool Upload(DeviceInformation device);

        /// <summary>發生在解析出 Hex 內容後。</summary>
        event HexParsedEventHandler                 HexParsed;
        /// <summary>發生在更新程序信息顯示時。</summary>
        event MessageShownEventHandler              MessageShown;
        /// <summary>發生在程序執行完成時。</summary>
        event ProcedureCompletedEventHandler        ProcedureCompleted;
        /// <summary>發生在程序進度改變時。</summary>
        event ProcedureProgressChangedEventHandler  ProcedureProgressChanged;
        /// <summary>發生在資料更新改變時。</summary>
        event UploadInformationChangedEventHandler  UploadInformationChanged;
    }
}
