﻿using System.Collections.Generic;

namespace PH.Bootloader.RS485.PIC32_2
{
    /// <summary>列舉所有裝置型態。</summary>
    public class DeviceType
    {
        /// <summary>PSU (PIC)。</summary>
        public static DeviceType PSU = new DeviceType(0, "PSU", "PSU\0");
        /// <summary>PFC (PIC)。</summary>
        public static DeviceType PFC = new DeviceType(1, "PFC", "PFC\0");
        /// <summary>SBox (PIC)。</summary>
        public static DeviceType SBox = new DeviceType(2, "SBox", "SBox");
        /// <summary>CSU (PIC)。</summary>
        public static DeviceType CSU = new DeviceType(3, "CSU", "CSU\0");

        #region Properties
        /// <summary>取得索引指標。</summary>
        public int      Index { get; private set; }
        /// <summary>取得文字描述。</summary>
        public string   Name { get; private set; }
        /// <summary>取得 Hex 辨識碼。</summary>
        public string   HexKey { get; private set; }
        #endregion

        #region Constructor
        /// <summary>初始化 DeviceType 類別的新執行個體。</summary>
        /// <param name="index"></param>
        /// <param name="name"></param>
        /// <param name="hexKey"></param>
        public DeviceType(int index, string name, string hexKey)
        {
            Index = index;
            Name = name;
            HexKey = hexKey;
        }
        #endregion

        #region Methods
        /// <summary>比較兩物件。</summary>
        /// <param name="target"></param>
        /// <returns></returns>
        public bool Equals(DeviceType target)
        {
            return Index == target.Index;
        }

        /// <summary>取得所有裝置定義。</summary>
        /// <returns></returns>
        public static IEnumerable<DeviceType> List
        {
            get
            {
                yield return PSU;
                yield return PFC;
                yield return SBox;
                yield return CSU;
            }
        }

        /// <summary>取得所有裝置定義名稱。</summary>
        /// <returns></returns>
        public static IEnumerable<string> ListDeviceName
        {
            get
            {
                yield return PSU.Name;
                yield return PFC.Name;
                yield return SBox.Name;
                yield return CSU.Name;
            }
        }

        /// <summary>根據索引值，解析出 PSU 通訊位址(站號)定義物件。</summary>
        /// <param name="rs485ID"></param>
        /// <returns></returns>
        public static DeviceType ParseByIndex(int index)
        {
            foreach (DeviceType device in List)
            {
                if (device.Index == index) {
                    return device;
                }
            }

            return null;
        }

        /// <summary>根據名稱，解析出 PSU 通訊位址(站號)定義物件。</summary>
        /// <param name="rs485ID"></param>
        /// <returns></returns>
        public static DeviceType ParseByName(string name)
        {
            foreach (DeviceType device in List)
            {
                if (device.Name == name) {
                    return device;
                }
            }

            return null;
        }
        #endregion
    }
}
