﻿using PH.Bootloader.RS485.PIC32_2.Protocol;
using PH.Bootloader.RS485.PIC32_2.Protocol.Command;
using PH.Bootloader.RS485.PIC32_2.Protocol.Command.AppCode;
using PH.Bootloader.RS485.PIC32_2.Protocol.Command.BootCode;

namespace PH.Bootloader.RS485.PIC32_2
{
    /// <summary>封裝 Microchip PIC32 啟動程序更新之類別(2.0版/瑱賢)。</summary>
    public partial class PIC32Bootloader2
    {
        private bool ExecuteCopyFlash(out int page)
        {
            FlashCopyCmd cmd = new FlashCopyCmd();
            bool isOK = protocolHandler.ExecuteBootCodeCommand(cmd);
            page = isOK ? cmd.Page : 0;

            return isOK;
        }

        private bool ExecuteEraseMemory(out int page)
        {
            EraseCmd cmd = new EraseCmd();
            bool isOK = protocolHandler.ExecuteBootCodeCommand(cmd);
            page = isOK ? cmd.Page : 0;

            return isOK;
        }

        private bool ExecuteGetVersion(byte devID, out string version)
        {
            ReadVersionCmd cmd = new ReadVersionCmd();
            bool isOK = protocolHandler.ExecuteAppCodeCommand(devID, cmd);
            version = isOK ? cmd.Version : string.Empty;

            return isOK;
        }

        private bool ExecuteTriggerDevice()
        {
            TriggerCmd cmd = new TriggerCmd();
            bool isSuccess = protocolHandler.ExecuteBootCodeCommand(cmd);
            if (isSuccess)
            {
                if (cmd.Ack1 == TriggerCmd.ACK1 && cmd.Ack2 == TriggerCmd.ACK2)
                {
                    return true;
                }
                else
                {
                    ChangeUpgradeStatusCallback(UpgradeStatusType.TriggerInvalidResponse);
                }
            }
            else
            {
                ChangeUpgradeStatusCallback(UpgradeStatusType.TriggerNoResponse);
            }

            return false;
        }

        private bool ExecuteUpgradeEnable(byte devID, UpgradeImageType imgType)
        {
            if (imgType == UpgradeImageType.AppCode ||
                imgType == UpgradeImageType.AppCodeByBootCode)
            {
                UpgradeAppCmd cmd = new UpgradeAppCmd();
                return protocolHandler.ExecuteAppCodeCommand(devID, cmd);
            }
            else if (imgType == UpgradeImageType.BootCode)
            {
                UpgradeBootloaderCmd cmd = new UpgradeBootloaderCmd();
                return protocolHandler.ExecuteAppCodeCommand(devID, cmd);
            }

            return false;
        }

        private bool ExecuteReset(byte devID, UpgradeImageType imgType)
        {
            if (imgType == UpgradeImageType.AppCode ||
                imgType == UpgradeImageType.AppCodeByBootCode)
            {
                ResetCmd cmd = new ResetCmd();
                return protocolHandler.ExecuteAppCodeCommandNotResp(devID, cmd);
            }

            return false;
        }

        private bool ExecuteVerify()
        {
            VerifyCmd cmd = new VerifyCmd();
            bool isSuccess = protocolHandler.ExecuteBootCodeCommand(cmd);
            if (isSuccess)
            {
                if (cmd.Ack == VerifyCmd.ACK)
                {
                    ChangeUpgradeStatusCallback(UpgradeStatusType.VerifyFinished);
                    return true;
                }
                else
                {
                    ChangeUpgradeStatusCallback(UpgradeStatusType.VerifyCommandFailed);
                    return false;
                }
            }

            ChangeUpgradeStatusCallback(UpgradeStatusType.VerifyNoResponse);
            return false;
        }
    }
}
