﻿using PH.Bootloader.RS485.PIC32_2.Protocol.Command;
using PH.Bootloader.RS485.PIC32_2.Protocol.Command.BootCode;

namespace PH.Bootloader.RS485.PIC32_2
{
    /// <summary>封裝 Microchip PIC32 啟動程序更新之類別(2.0版/瑱賢)。</summary>
    public partial class PIC32Bootloader2
    {
        private bool CopyFlashProcess()
        {
            bool isSuccess = false;
            int totalPage;

            if (ExecuteCopyFlash(out totalPage))
            {
                progressStep = 1.0F / totalPage;
                progressPercentage = 0;

                IsProcedureRunning = true;
                ChangeUpgradeProgressCallback(UpgradeProgressType.CopyFlash, 0);
                ChangeUpgradeStatusCallback(UpgradeStatusType.MemoryCopying);

                byte ack;
                int copyPage = 0;
                while (protocolHandler.ReadByteAtCertainTime(out ack, commSyncTime)) {
                    if (ack == FlashCopyCmd.COPY_DONE && copyPage == totalPage) {
                        isSuccess = true;
                        break;
                    } else {
                        progressPercentage += progressStep;
                        if (progressPercentage >= 1.0F) {
                            progressPercentage = 1.0F;
                        }
                        ChangeUpgradeProgressCallback(UpgradeProgressType.CopyFlash, (int)(progressPercentage*100.0F));

                        copyPage++;
                        if (copyPage > totalPage) {
                            ChangeUpgradeStatusCallback(UpgradeStatusType.MemoryCopySizeErorr);
                            break;
                        }
                    }
                }

                IsProcedureRunning = false;
                ChangeUpgradeProgressCallback(UpgradeProgressType.CopyFlash, 0);

                if (isSuccess)
                {
                    if (protocolHandler.ReadByteAtCertainTime(out ack, commSyncTime) && 
                        ack == VerifyCmd.ACK)
                    {
                        ChangeUpgradeStatusCallback(UpgradeStatusType.MemoryCopyFinished);
                        return true;
                    }
                    else
                    {
                        ChangeUpgradeStatusCallback(UpgradeStatusType.MemoryCopyNoVerified);
                    }
                }
                else
                {
                    ChangeUpgradeStatusCallback(UpgradeStatusType.MemoryCopyNotDone);
                }
            }
            else
            {
                ChangeUpgradeStatusCallback(UpgradeStatusType.MemoryCopyCommandFailed);
            }

            return false;
        }

        private bool EraseMemoryProcess()
        {
            bool isSuccess = false;

            int totalPage;
            if (ExecuteEraseMemory(out totalPage))
            {
                progressStep = totalPage <= 0 ? 0 : 1.0F / totalPage;
                progressPercentage = 0;

                IsProcedureRunning = true;
                ChangeUpgradeProgressCallback(UpgradeProgressType.EraseFlash, 0);
                ChangeUpgradeStatusCallback(UpgradeStatusType.FlashErasing);

                int erasedPage = 0;
                byte ack;
                while (protocolHandler.ReadByteAtCertainTime(out ack, commSyncTime))
                {
                    if (ack == EraseCmd.ERASE_DONE &&
                        erasedPage == totalPage)
                    {
                        ChangeUpgradeStatusCallback(UpgradeStatusType.EraseFlashFinished);
                        isSuccess = true;
                        break;
                    }
                    else
                    {
                        progressPercentage += progressStep;
                        if (progressPercentage >= 1.0F) {
                            progressPercentage = 1.0F;
                        }
                        ChangeUpgradeProgressCallback(UpgradeProgressType.EraseFlash, (int)(progressPercentage*100.0F));

                        erasedPage++;
                        if (erasedPage > totalPage) {
                            ChangeUpgradeStatusCallback(UpgradeStatusType.EraseFlashSizeError);
                            break;
                        }
                    }
                }

                IsProcedureRunning = true;
                ChangeUpgradeProgressCallback(UpgradeProgressType.EraseFlash, 0);

                return isSuccess;
            }
            else
            {
                ChangeUpgradeStatusCallback(UpgradeStatusType.EraseFlashCommandFailed);
            }

            return false;
        }
    }
}
