﻿using PH.Bootloader.RS485.PIC32_2.Protocol.Command.BootCode;

namespace PH.Bootloader.RS485.PIC32_2.Protocol
{
    /// <summary>啟動程式碼封包之處理類別。</summary>
    class BootCodePacketFactory
    {
        private const byte  TX_PACKET_HEAD1 = 0x55;
        private const byte  TX_PACKET_HEAD2 = 0xAA;
        private const int   TX_PACKET_SIZE  = 5;

        /// <summary>產生資料封包陣列。</summary>
        /// <param name="cmd">啟動程式碼指令。</param>
        /// <returns>傳送的資料陣列。</returns>
        public static byte[] Create(IBootCommand cmd)
        {
            byte[] packetArray = new byte[TX_PACKET_SIZE];
            packetArray[0] = TX_PACKET_HEAD1;
            packetArray[1] = TX_PACKET_HEAD2;
            packetArray[2] = cmd.Code;
            packetArray[3] = TX_PACKET_HEAD2;
            packetArray[4] = TX_PACKET_HEAD1;

            return packetArray;
        }

        /// <summary>預估回應的封包長度。</summary>
        /// <param name="cmd">啟動程式碼指令。</param>
        /// <returns>預估回應長度。</returns>
        public static int EstimateResponsePacketLength(IBootCommand cmd)
        {
            return cmd.EstimateAckLength;
        }

        /// <summary>解析封包格式內容。</summary>
        /// <param name="recvDataArray">接收資料暫存器。</param>
        /// <param name="recvLength">接收資料長度。</param>
        /// <param name="cmd">回傳指令內容。</param>
        /// <returns>是否解析成功。</returns>
        public static bool Parse(byte[] recvDataArray, int recvLength, ref IBootCommand cmd)
        {
            return cmd.Parse(recvDataArray, recvLength);
        }
    }
}
