﻿
using PH.Bootloader.RS485.DSP;
using PH.Bootloader.RS485.PIC32;

namespace PH.Bootloader.RS485
{
    /// <summary>RS-485  啟動程序安裝工具工廠類別。</summary>
    class RS485BootloaderFactory
    {
        private static readonly object thisLock = new object();
        private static RS485BootloaderFactory instance;

        /// <summary>取得實體物件。</summary>
        /// <returns></returns>
        public static RS485BootloaderFactory GetInstance()
        {
            if (instance == null)
            {
                lock (thisLock) {
                    if (instance == null) {
                        instance = new RS485BootloaderFactory();
                    }
                }
            }

            return instance;
        }

        private DspBootloader dspBootloader;
        private PicBootloader picBootloader;

        private RS485BootloaderFactory()
        {
            dspBootloader = new DspBootloader();
            picBootloader = new PicBootloader();
        }

        /// <summary>根據裝置型態，創建所屬的 Bootloader。</summary>
        /// <param name="deviceType"></param>
        /// <returns></returns>
        public IBootloader Create(DeviceType deviceType)
        {
            if (deviceType.Equals(DeviceType.DSP_PSU) ||
                deviceType.Equals(DeviceType.DSP_PFC))
            {
                if (dspBootloader.IsProcedureRunning) {
                    dspBootloader.Abort();
                }
                
                return dspBootloader;
            }
            else
            {
                if (picBootloader.IsProcedureRunning) {
                    picBootloader.Abort();
                }

                return picBootloader;
            }
        }
    }
}
