using System.Collections.Generic;
using System.Collections.ObjectModel;

namespace PhihongEv.Lib.Model
{
    //public enum NetworkOption
    //{
    //    Unknown,
    //    None,
    //    Ethernet,
    //    Ble,
    //    EthWiFi,
    //    Eth4G,
    //    EthBle,
    //    EthWiFi4G,
    //    WiFi
    //}

    public sealed partial class NetworkOption : ModelNameCodeModel
    {
        public static NetworkOption Unknown = new NetworkOption("Unknown", "Unknown", addRegister: false);

        public static NetworkOption None = new NetworkOption("0", "None");

        public static NetworkOption Ethernet = new NetworkOption(
            "E", "Ethernet only", 
            supportEthernet: true);

        public static NetworkOption Ble = new NetworkOption(
            "B", "Bluetooth only" , 
            supportBluetooth: true);

        public static NetworkOption EthWiFi = new NetworkOption(
            "W", "Ethernet + WiFi", 
            supportEthernet: true, supportWiFi: true);

        public static NetworkOption Eth4G = new NetworkOption(
            "T", "Ethernet + 4G",
            supportEthernet: true, supportTelecom: true);

        public static NetworkOption EthBle = new NetworkOption(
            "U", "Ethernet + Bluetooth",
            supportEthernet: true, supportBluetooth: true);

        public static NetworkOption EthWiFi4G = new NetworkOption(
            "D", "Ethernet + WiFi + 4G",
            supportEthernet: true, supportWiFi: true, supportTelecom: true);

        public static NetworkOption EthWiFi4G_antenna = new NetworkOption(
            "A", "Ethernet + WiFi external antenna + 4G",
            supportEthernet: true, supportWiFi: true, supportTelecom: true );

        public static NetworkOption WiFi = new NetworkOption(
            "F", "WiFi only", 
            supportWiFi: true);
    }

    public sealed partial class NetworkOption : ModelNameCodeModel
    {
        public static ReadOnlyCollection<NetworkOption> List => new ReadOnlyCollection<NetworkOption>(_List);

        private static List<NetworkOption> _List { get; set; }

        public bool SupportEthernet { get; private set; }
        public bool SupportWiFi { get; private set; }
        public bool SupportTelecom { get; private set; }
        public bool SupportBluetooth { get; private set; }

        internal NetworkOption(string code, string description,
            bool supportEthernet = false, bool supportWiFi = false,
            bool supportTelecom = false, bool supportBluetooth = false, 
            bool addRegister = true) : base(code, description)
        {
            SupportEthernet = supportEthernet;
            SupportWiFi = supportWiFi;
            SupportTelecom = supportTelecom;
            SupportBluetooth = supportBluetooth;

            if (_List == null)
            {
                _List = new List<NetworkOption>();
            }

            if (!addRegister)
            {
                return;
            }

            _List.Add(this);
        }
    }
}