﻿using System;
using System.Text;

namespace PhihongEv.Lib
{
    public class SystemID
    {
        public ModelName ModelName { get; private set; }
        public string SerialNumber { get; private set; }

        public string sourceString { get; private set; }

        public static bool TryParse(byte[] byteData, out SystemID connectInfo)
        {
            connectInfo = null;
            if (byteData == null || byteData.Length < 14)
                return false;

            string systemIDString = null;
            try
            {
                systemIDString = Encoding.ASCII.GetString(byteData,0,byteData.Length);
            }
            catch
            {
                return false;
            }

            if (!string.IsNullOrEmpty(systemIDString))
            {
                return TryParse(systemIDString, out connectInfo);
            }
            return false;
        }

        public static bool TryParse(string byteString, out SystemID connectInfo)
        {
            connectInfo = null;
            SystemID tmp = new SystemID();

            tmp.sourceString = byteString;

            if (tmp.sourceString.Length < 14)
                return false;

            if (!ModelName.TryParse(tmp.sourceString.Substring(0, 14), out var model))
                return false;

            tmp.ModelName = model;
            tmp.SerialNumber = tmp.sourceString.Substring(14);
            connectInfo = tmp;

            return true;
        }

        public static bool TryLooseParse(string byteString, out SystemID connectInfo)
        {
            connectInfo = null;

            SystemID tmp = new SystemID();

            tmp.sourceString = byteString;

            if (tmp.sourceString.Length < 14)
                return false;

            if (!ModelName.TryLooseParse(tmp.sourceString.Substring(0, 14), out var model))
            {
                return false;
            }

            tmp.ModelName = model;
            tmp.SerialNumber = tmp.sourceString.Substring(14);

            connectInfo = tmp;

            return true;
        }

        public string ToWifiPwd()
        {
            return SerialNumber + ModelName.ToString();
        }

        public int GetConnectorViewIndex(int arrayIndex) => ModelName.GetConnectorViewIndex(arrayIndex);
        public int GetConnectorArrayIndex(int viewIndex) => ModelName.GetConnectorArrayIndex(viewIndex);

        public override string ToString()
        {
            return sourceString;
        }

        public override bool Equals(object obj)
        {
            return obj.ToString() == this.ToString();
        }

        public override int GetHashCode()
        {
            return this.ToString().GetHashCode();
        }

        public static bool operator ==(SystemID a, SystemID b)
        {
            string stra = null, strb = null;

            if (Object.Equals(a, null))
                stra = null;
            else
                stra = a.ToString();

            if (Object.Equals(b, null))
                strb = null;
            else
                strb = b.ToString();
            return string.Equals(a, b);
        }

        public static bool operator !=(SystemID a, SystemID b)
        {
            string stra = null, strb = null;

            if (Object.Equals(a, null))
                stra = null;
            else
                stra = a.ToString();

            if (Object.Equals(b, null))
                strb = null;
            else
                strb = b.ToString();
            return !string.Equals(a, b);
        }
    }

}