﻿using System.Windows.Forms;
using Net.Core.Communication.Serial.Sync;
using PH.Bootloader.RS485;
using PH.Bootloader.RS485.DSP.IntelHex;
using PH.Bootloader.RS485.DSP.Procedure;

namespace DSPBootloader
{
    public partial class Form1
    {
        /// <summary>發生在通訊埠狀態改變時。</summary>
        /// <param name="isOpen"></param>
        private void SyncComm_StatusChanged(SyncCommStatus status)
        {
            isOpenComm = status == SyncCommStatus.Opened;

            if (isOpenComm) {
                bootloader.AssignComPort(syncComm);
            } else {
                bootloader.Abort();
            }

            ControlUI(isOpenComm, true);
        }

        private void Bootloader_MetaBlockParsed(object sender, MetaBlockContent metaBlockContent)
        {
            MetaBlockVerLabel.Text = metaBlockContent.MetaBlockVersion;
            SoftwareVerLabel.Text = metaBlockContent.SoftwareVersion;
            StartAddressLabel.Text = metaBlockContent.StartAddressText;
            StopAddressLabel.Text = metaBlockContent.StopAddressText;
        }

        private void Bootloader_HexRecordParsed(object sender, IntelHexRecord[] hexRecords)
        {
            HexFileRawData.Items.Clear();
            foreach (IntelHexRecord record in hexRecords) {
                HexFileRawData.Items.Add(record.ToString());
            }
        }

        private void Bootloader_MessageShown(object sender, string message)
        {
            ProcessInfo.AppendText(message);
            //ProcessInfo.TopIndex = ProcessInfo.Items.Count - 1;
        }

        private void Bootloader_ProcedureProgressChanged(object sender, ProcedureType procType, int progressPercentage)
        {
            if (procType == ProcedureType.Upload)
            {
                toolStripStatusLabel1.Text = progressHintIndex == 0 ? "Now loading" : toolStripStatusLabel1.Text + ".";
                progressHintIndex = progressHintIndex >= 4 ? 0 : progressHintIndex + 1;

                toolStripStatusLabel5.Text = string.Format("{0}%", progressPercentage);
                Upgrade_Progress_Bar.Value = progressPercentage;
            }
        }

        private void Bootloader_ProcedureCompleted(object sender, ProcedureType procType, BootloadResult result)
        {
            if (procType == ProcedureType.CheckDevice)
            {
                if (isOpenComm) {
                    CheckDeviceBTN.Enabled = true;
                }
            }
            else if (procType == ProcedureType.Upload)
            {
                if (isOpenComm) {
                    ControlUI(true, false);
                }
                
                MessageBox.Show(string.Format("Update firmware {0}\n", result == BootloadResult.Uploaded ? "successfully" : "failed"), "Completed", MessageBoxButtons.OK, MessageBoxIcon.Information);

                progressHintIndex = 0;
                toolStripStatusLabel1.Text = string.Empty;
                toolStripStatusLabel5.Text = string.Empty;
                Upgrade_Progress_Bar.Value = 0;

                CheckDeviceBTN.PerformClick();
            }
        }
    }
}
