﻿using System.IO.Ports;
using System.Reflection;
using System.Windows.Forms;
using Net.Core.Communication.Serial.Sync;
using PH.Bootloader.RS485;
using PH.Bootloader.RS485.Communication;
using PH.Bootloader.RS485.DSP;

namespace DSPBootloader
{
    public partial class Form1 : Form
    {
        #region Fields
        private DeviceInformation   targetDevice;
        private ISyncCommunicationPlus syncComm;
        private bool                isOpenComm;
        //
        private DspBootloader       bootloader;
        private int                 progressHintIndex;
        #endregion

        #region Methods : System
        /// <summary>視窗建構子。</summary>
        public Form1()
        {
            InitializeComponent();
        }

        /// <summary>視窗載入。</summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form1_Load(object sender, System.EventArgs e)
        {
            CheckForIllegalCrossThreadCalls = false;
            Text = Text + string.Format(" {0}{1}", "V", Assembly.GetEntryAssembly().GetName().Version);

            targetDevice = DeviceInformation.C1P1;

            syncComm = new SerialPortHandler(300);
            syncComm.TxSilenceEnable = false;
            syncComm.StatusChanged += SyncComm_StatusChanged; ;

            bootloader = new DspBootloader();
            bootloader.Name = "DSP Bootloader";
            bootloader.MetaBlockParsed += Bootloader_MetaBlockParsed;
            bootloader.HexRecordParsed += Bootloader_HexRecordParsed;
            bootloader.MessageShown += Bootloader_MessageShown;
            bootloader.ProcedureProgressChanged += Bootloader_ProcedureProgressChanged;
            bootloader.ProcedureCompleted += Bootloader_ProcedureCompleted;

            CommPortOption.Items.AddRange(SerialPort.GetPortNames());
            if (CommPortOption.Items.Count > 0) {
                CommPortOption.SelectedIndex = 0;
            }

            DeviceOption.Items.AddRange(DeviceInformation.GetNameList(DeviceType.DSP_PSU));
            DeviceOption.SelectedIndex = 0;

            OpenCommBTN.Click += OpenCommBTN_Click;
            DeviceOption.SelectedIndexChanged += DeviceOption_SelectedIndexChanged;
            CheckDeviceBTN.Click += CheckDeviceBTN_Click;
            LoadFileBTN.Click += LoadFileBTN_Click;
            UpgradeBTN.Click += UpgradeBTN_Click;
        }
        #endregion

        #region Methods : Private
        private void ControlUI(bool isCommReady, bool includeButtonName)
        {
            if (includeButtonName) {
                OpenCommBTN.Text = isCommReady ? "Close" : "Open";
            }
            DeviceOption.Enabled = isCommReady;
            CheckDeviceBTN.Enabled = isCommReady;
            LoadFileBTN.Enabled = isCommReady;
            UpgradeBTN.Enabled = isCommReady;
            CanAgentChecker.Enabled = isCommReady;
        }
        #endregion

        #region Methods : Callback
        /// <summary>發生在開啟/關閉通訊埠按鍵觸發時。</summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OpenCommBTN_Click(object sender, System.EventArgs e)
        {
            if (!isOpenComm)
            {
                syncComm.Open((string)CommPortOption.SelectedItem, 115200);
            }
            else
            {
                syncComm.Close();
            }
        }

        /// <summary>發生在目標裝置下拉選項改變時。</summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void DeviceOption_SelectedIndexChanged(object sender, System.EventArgs e)
        {
            targetDevice = DeviceInformation.ParseByName((string)DeviceOption.SelectedItem);
        }

        /// <summary>發生在裝置檢查按鍵觸發時。</summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CheckDeviceBTN_Click(object sender, System.EventArgs e)
        {
            if (bootloader.CheckDevice(targetDevice))
            {
                CheckDeviceBTN.Enabled = false;
                ProcessInfo.Clear();
            }
        }

        /// <summary>發生在載入 Hex 檔按鍵觸發時。</summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LoadFileBTN_Click(object sender, System.EventArgs e)
        {
            HexFileRawData.Items.Clear();

            OpenFileDialog openFileDialog = new OpenFileDialog();
            openFileDialog.Filter = "hex files (*.hex*)|*.hex*|All files (*.*)|*.*";
            openFileDialog.FilterIndex = 1;
            openFileDialog.RestoreDirectory = true;

            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                LoadFilePath.Text = openFileDialog.FileName;

                bootloader.ParseHexToMemory(targetDevice, openFileDialog.FileName);
            }
        }

        /// <summary>發生在更新按鍵觸發時。</summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void UpgradeBTN_Click(object sender, System.EventArgs e)
        {
            bootloader.IsCanAgent = CanAgentChecker.Checked;
            if (bootloader.Upload(targetDevice))
            {
                DeviceOption.Enabled = false;
                CheckDeviceBTN.Enabled = false;
                LoadFileBTN.Enabled = false;
                UpgradeBTN.Enabled = false;
                CanAgentChecker.Enabled = false;
            }
        }
        #endregion
    }
}
