﻿using System;
using System.Linq;
using System.Reflection;
using System.Windows.Forms;
using FirmwareHeaderBuilder.Controller;
using FirmwareHeaderBuilder.Model;
using FirmwareHeaderBuilder.Model.ModelNaming;
using PhihongEv.Lib.Model;

namespace FirmwareHeaderBuilder
{
    public partial class Form1 : Form
    {
        private HeaderBuilder   headerBuilder;
        private OpenFileDialog  openFileDialog;

        public Form1()
        {
            InitializeComponent();

            InitializeMyComponent();
            InitializeUI();
        }

        private void OptImageType_SelectedIndexChanged(object sender, EventArgs e)
        {
            ImageType imgType = ImageType.Parse((ushort)optImageType.SelectedIndex);
            headerBuilder.SelectImageType(imgType.Index);
            openFileDialog.FilterIndex = imgType.Filter.Index + 1; // Base 1??
        }

        private void BtnBrowse_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                lbFilePath.Text = openFileDialog.FileName;

                if (headerBuilder.CheckHeaderExist(lbFilePath.Text))
                {
                    if (MessageBox.Show("已存在檔頭資訊，合併後將被複寫，是否繼續進行?",　"載入", MessageBoxButtons.OKCancel, MessageBoxIcon.Warning) == DialogResult.OK)
                    {
                        ErrorCode errCode = headerBuilder.ImportImage(lbFilePath.Text, true);
                        PostErrorCode(errCode);
                    }
                    else
                    {
                        headerBuilder.ClearImage();
                        lbFilePath.Text = string.Empty;
                    }
                }
                else
                {
                    ErrorCode errCode = headerBuilder.ImportImage(lbFilePath.Text);
                    PostErrorCode(errCode);
                }
            }
        }

        private void BtnMerge_Click(object sender, EventArgs e)
        {
            ErrorCode errCode = headerBuilder.InputModelName(txtModelName.Text); 

            if (errCode == ErrorCode.NoError)
            {
                Properties.Settings.Default.type = optType.SelectedIndex;
                Properties.Settings.Default.configuration = optInputConfiguration.SelectedIndex;
                Properties.Settings.Default.safety = optSafetyRegulation.SelectedIndex;
                Properties.Settings.Default.connector1 = optConnector1.SelectedIndex;
                Properties.Settings.Default.connector2 = optConnector2.SelectedIndex;
                Properties.Settings.Default.connector3 = optConnector3.SelectedIndex;
                Properties.Settings.Default.network = optNetworkOption.SelectedIndex;
                Properties.Settings.Default.generation = optGeneration.SelectedIndex;
                Properties.Settings.Default.imagetype = optImageType.SelectedIndex;
                Properties.Settings.Default.ratepower = txtRatedPower.Text;
                Properties.Settings.Default.customization = txtCustomization.Text;
                Properties.Settings.Default.Save();

                errCode = headerBuilder.Build();
                PostErrorCode(errCode);
            }
            else
            {
                PostErrorCode(errCode);
            }
        }

        private void InitializeMyComponent()
        {
            headerBuilder = new HeaderBuilder();

            optType.SelectedIndexChanged += ModuleNaming_SelectedIndexChanged; 
            optInputConfiguration.SelectedIndexChanged += ModuleNaming_SelectedIndexChanged;
            optSafetyRegulation.SelectedIndexChanged += ModuleNaming_SelectedIndexChanged;
            //optRatedPower.SelectedIndexChanged += ModuleNaming_SelectedIndexChanged;
            txtRatedPower.TextChanged += ModuleNaming_SelectedIndexChanged;
            optConnector1.SelectedIndexChanged += ModuleNaming_SelectedIndexChanged;
            optConnector2.SelectedIndexChanged += ModuleNaming_SelectedIndexChanged;
            optConnector3.SelectedIndexChanged += ModuleNaming_SelectedIndexChanged;
            optNetworkOption.SelectedIndexChanged += ModuleNaming_SelectedIndexChanged;
            optGeneration.SelectedIndexChanged += ModuleNaming_SelectedIndexChanged;
            txtCustomization.TextChanged += ModuleNaming_SelectedIndexChanged;

            optImageType.SelectedIndexChanged += OptImageType_SelectedIndexChanged;
            optImageType.Items.Clear();
            optImageType.Items.AddRange(ImageType.ListText.ToArray());

            btnBrowse.Click += BtnBrowse_Click;
            btnMerge.Click += BtnMerge_Click;

            openFileDialog = new OpenFileDialog();
            openFileDialog.Filter = OpenDialogFilter.GetFilterRuleText();
        }

        private void ModuleNaming_SelectedIndexChanged(object sender, EventArgs e)
        {
            MajorType majorType = MajorType.List.FirstOrDefault(x=>x.Description == (string)optType.SelectedItem);
            InputConfig inputConfig = InputConfig.List.FirstOrDefault(x => x.Description == (string)optInputConfiguration.SelectedItem);
            SafetyRegulation safetyRegulation = SafetyRegulation.List.FirstOrDefault(x => x.Description == (string)optSafetyRegulation.SelectedItem);
            //RatedPower ratedPower = RatedPower.Parse((string)optRatedPower.SelectedItem);
            ConnectorType socketType1 = ConnectorType.List.FirstOrDefault(x => x.Description == (string)optConnector1.SelectedItem);
            ConnectorType socketType2 = ConnectorType.List.FirstOrDefault(x => x.Description == (string)optConnector2.SelectedItem);
            ConnectorType socketType3 = ConnectorType.List.FirstOrDefault(x => x.Description == (string)optConnector3.SelectedItem);
            NetworkOption networkOption = NetworkOption.List.FirstOrDefault(x => x.Description == (string)optNetworkOption.SelectedItem);
            Generation generation = Generation.List.FirstOrDefault(x => x.Description == (string)optGeneration.SelectedItem);

            if (string.IsNullOrEmpty(txtRatedPower.Text))
            {
                MessageBox.Show("請輸入額定功率代碼", "ModuleNaming", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                txtRatedPower.Text = "000";
                return;
            }

            if (majorType != null &&
                inputConfig != null &&
                safetyRegulation != null &&
                socketType1 != null &&
                socketType2 != null &&
                socketType3 != null &&
                //ratedPower != null &&
                networkOption != null &&
                generation != null)
            {
                txtModelName.Text = majorType.Code + inputConfig.Code + safetyRegulation.Code + txtRatedPower.Text + socketType1.Code + socketType2.Code + socketType3.Code + networkOption.Code + generation.Code + txtCustomization.Text;
            }
        }

        private void InitializeUI()
        {
            Text = Text + string.Format(" V{0}", Assembly.GetEntryAssembly().GetName().Version);

            optType.Items.AddRange(MajorType.List.Select(x=>x.Description).ToArray());
            optInputConfiguration.Items.AddRange(InputConfig.List.Select(x => x.Description).ToArray());
            optSafetyRegulation.Items.AddRange(SafetyRegulation.List.Select(x => x.Description).ToArray());
            //optRatedPower.Items.AddRange(RatedPower.ListName.ToArray());
            optConnector1.Items.AddRange(ConnectorType.List.Select(x => x.Description).ToArray());
            optConnector2.Items.AddRange(ConnectorType.List.Select(x => x.Description).ToArray());
            optConnector3.Items.AddRange(ConnectorType.List.Select(x => x.Description).ToArray());
            optNetworkOption.Items.AddRange(NetworkOption.List.Select(x => x.Description).ToArray());
            optGeneration.Items.AddRange(Generation.List.Select(x => x.Description).ToArray());

            optType.SelectedIndex = Properties.Settings.Default.type;
            optInputConfiguration.SelectedIndex = Properties.Settings.Default.configuration;
            optSafetyRegulation.SelectedIndex = Properties.Settings.Default.safety;
            //optRatedPower.SelectedIndex = 0;
            optConnector1.SelectedIndex = Properties.Settings.Default.connector1;
            optConnector2.SelectedIndex = Properties.Settings.Default.connector2;
            optConnector3.SelectedIndex = Properties.Settings.Default.connector3;
            optNetworkOption.SelectedIndex = Properties.Settings.Default.network;
            optGeneration.SelectedIndex = Properties.Settings.Default.generation;
            optImageType.SelectedIndex = Properties.Settings.Default.imagetype;

            txtCustomization.Text = Properties.Settings.Default.customization;
            txtRatedPower.Text = Properties.Settings.Default.ratepower;
        }

        private void PostErrorCode(ErrorCode code)
        {
            switch(code)
            {
                case ErrorCode.BuiltFileError:
                    MessageBox.Show("創建檔案失敗", "執行", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    break;
                case ErrorCode.BuiltSuccess:
                    MessageBox.Show("創建完成", "執行", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    break;
                case ErrorCode.FileNotExist:
                    MessageBox.Show("映像檔不存在", "執行", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.FilePathEmpty:
                    MessageBox.Show("尚未選擇映像檔", "執行", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ImageDataNotReady:
                    MessageBox.Show("尚未選擇映像檔", "執行", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ImageTypeIndexNotExist:
                    MessageBox.Show("未定義的映像檔型態", "執行", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    break;
                case ErrorCode.MemoryAllocationError:
                    MessageBox.Show("記憶體配置時發生錯誤", "執行", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    break;
                case ErrorCode.ModelNameEmpty:
                    MessageBox.Show("尚未輸入 Model Name", "執行", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case ErrorCode.ModelNameInputTypeError:
                    MessageBox.Show("輸入的 Model Name 出現不合法的字元", "執行", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    break;
                case ErrorCode.ModelNameLenError:
                    MessageBox.Show("輸入的  Model Name 長度須為 14", "執行", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    break;
                case ErrorCode.ReadFileError:
                    MessageBox.Show("讀取映像檔時發生錯誤", "執行", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    break;

                case ErrorCode.NoError:
                default:
                    break;
            }
        } 
    }
}
