﻿
namespace FirmwareHeaderBuilder.Model
{
    /// <summary>CRC-32 運算類別。</summary>
    /// 獨體模式。
    class CRC32
    {
        private static CRC32 inatance;

        /// <summary>取得實體。</summary>
        /// <returns></returns>
        public static CRC32 GetInatance()
        {
            if (inatance == null) {
                inatance = new CRC32();
            }

            return inatance;
        }

        private ulong[] crc32Table;

        /// <summary>初始化 CRC32 類別的新執行個體。</summary>
        private CRC32()
        {
            crc32Table = new ulong[256];
            GetCRC32Table();
        }

        /// <summary>計算。</summary>
        /// <param name="dataArray"></param>
        /// <param name="length"></param>
        /// <returns></returns>
        public ulong Calculate(byte[] dataArray, int length)
        {
            ulong crc = 0xFFFFFFFF;

            for (int i = 0; i < length; i++) {
                crc = (crc >> 8) ^ crc32Table[(crc & 0xFF) ^ dataArray[i]];
            }

            return crc ^ 0xFFFFFFFF;
        }

        private void GetCRC32Table()
        {
            ulong crc;
            int i, j;

            for (i = 0; i < 256; i++)
            {
                crc = (ulong)i;
                for (j = 8; j > 0; j--)
                {
                    if ((crc & 1) == 1) {
                        crc = (crc >> 1) ^ 0xEDB88320;
                    } else {
                        crc >>= 1;
                    }    
                }
                crc32Table[i] = crc;
            }
        }
    }
}
