﻿using System.Collections.Generic;

namespace FirmwareHeaderBuilder.Model
{
    /// <summary>列舉所有映像檔型態。</summary>
    public class ImageType
    {
        /// <summary>CSU 3.0 bootloader (uboot)。</summary>
        public static ImageType CSU30_Bootloader                = new ImageType(0, "CSU 3.0 bootloader (uboot)", new byte[] { 0x10, 0x00, 0x00, 0x01 }, OpenDialogFilter.ALL);
        /// <summary>CSU 3.0 kernel configuration (dtb)。</summary>
        public static ImageType CSU30_KernelConfig              = new ImageType(1, "CSU 3.0 kernel configuration (dtb)", new byte[] { 0x10, 0x00, 0x00, 0x02 }, OpenDialogFilter.DTB);
        /// <summary>CSU 3.0 kernel image (zImage)。</summary>
        public static ImageType CSU30_KernelImage               = new ImageType(2, "CSU 3.0 kernel image (zImage)", new byte[] { 0x10, 0x00, 0x00, 0x03 }, OpenDialogFilter.ALL);
        /// <summary>CSU 3.0 root file system。</summary>
        public static ImageType CSU30_RootFileSystem            = new ImageType(3, "CSU 3.0 root file system", new byte[] { 0x10, 0x00, 0x00, 0x04 }, OpenDialogFilter.ALL);
        /// <summary>CSU 3.0 user configuration (bin)。</summary>
        public static ImageType CSU30_UserConfig                = new ImageType(4, "CSU 3.0 user configuration (bin)", new byte[] { 0x10, 0x00, 0x00, 0x05 }, OpenDialogFilter.BIN);
        /// <summary>CSU 3.0 primary controller。</summary>
        public static ImageType CSU30_PrimaryController         = new ImageType(5, "CSU 3.0 primary controller", new byte[] { 0x10, 0x00, 0x00, 0x06 }, OpenDialogFilter.ALL);
        /// <summary>CSU 3.0 CCS board bootloader (uboot)。</summary>
        public static ImageType CSU30_CCS_BoardBootloader       = new ImageType(6, "CSU 3.0 CCS board bootloader (uboot)", new byte[] { 0x10, 0x00, 0x00, 0x07 }, OpenDialogFilter.ALL);
        /// <summary>CSU 3.0 CCS board kernel configuration (dtb)。</summary>
        public static ImageType CSU30_CCS_BoardKernelConfig     = new ImageType(7, "CSU 3.0 CCS board kernel configuration (dtb)", new byte[] { 0x10, 0x00, 0x00, 0x08 }, OpenDialogFilter.DTB);
        /// <summary>CSU 3.0 CCS board kernel image (zImage)。</summary>
        public static ImageType CSU30_CCS_BoardKernelImage      = new ImageType(8, "CSU 3.0 CCS board kernel image (zImage)", new byte[] { 0x10, 0x00, 0x00, 0x09 }, OpenDialogFilter.ALL);
        /// <summary>CSU 3.0 CCS board file system (ramdisk.gz)。</summary>
        public static ImageType CSU30_CCS_BoardFileSystem       = new ImageType(9, "CSU 3.0 CCS board file system (ramdisk.gz)", new byte[] { 0x10, 0x00, 0x00, 0x0A }, OpenDialogFilter.GZ);
        /// <summary>CSU 3.0 CAHdeMO board。</summary>
        public static ImageType CSU30_CAHdeMO_Board             = new ImageType(10, "CSU 3.0 CAHdeMO board", new byte[] { 0x10, 0x00, 0x00, 0x0B }, OpenDialogFilter.HEX);
        /// <summary>CSU 3.0 GB board。</summary>
        public static ImageType CSU30_GB_Board                  = new ImageType(11, "CSU 3.0 GB board", new byte[] { 0x10, 0x00, 0x00, 0x0C }, OpenDialogFilter.HEX);
        /// <summary>Relay control board。</summary>
        public static ImageType RelayControlBoard               = new ImageType(12, "Relay control board", new byte[] { 0x10, 0x00, 0x00, 0x0D }, OpenDialogFilter.HEX);
        /// <summary>Fan control board。</summary>
        public static ImageType FanControlBoard                 = new ImageType(13, "Fan control board", new byte[] { 0x10, 0x00, 0x00, 0x0E }, OpenDialogFilter.HEX);
        /// <summary>LCM。</summary>
        public static ImageType LCM                             = new ImageType(14, "LCM", new byte[] { 0x10, 0x00, 0x00, 0x0F }, OpenDialogFilter.HEX);
        /// <summary>F750 PSU primary controller。</summary>
        public static ImageType Infy_PFC_PrimaryController      = new ImageType(15, "Infy PFC primary controller", new byte[] { 0x10, 0x00, 0x00, 0x10 }, OpenDialogFilter.HEX);
        /// <summary>F750 PSU secondary controller。</summary>
        public static ImageType Infy_DD_SecondaryController     = new ImageType(16, "Infy DD secondary controller", new byte[] { 0x10, 0x00, 0x00, 0x11 }, OpenDialogFilter.HEX);
        /// <summary>F950 PSU primary controller。</summary>
        public static ImageType F950_PSU_PrimaryController      = new ImageType(17, "F950 PSU primary controller", new byte[] { 0x10, 0x00, 0x00, 0x12 }, OpenDialogFilter.HEX);
        /// <summary>F950 PSU secondary controller。</summary>
        public static ImageType F950_PSU_SecondaryController    = new ImageType(18, "F950 PSU secondary controller", new byte[] { 0x10, 0x00, 0x00, 0x13 }, OpenDialogFilter.HEX);
        /// <summary>LED control board。</summary>
        public static ImageType LED_control_board               = new ImageType(19, "LED control board", new byte[] { 0x10, 0x00, 0x00, 0x14 }, OpenDialogFilter.HEX);
        /// <summary>"LCM APK。</summary>
        public static ImageType LCM_APK                         = new ImageType(20, "LCM APK", new byte[] { 0x10, 0x00, 0x00, 0x15 }, OpenDialogFilter.ALL);
        /// <summary>LCM advertise video。</summary>
        public static ImageType LCM_Advertise_Video             = new ImageType(21, "LCM advertise video", new byte[] { 0x10, 0x00, 0x00, 0x16 }, OpenDialogFilter.ALL);
        /// <summary>CSU MLO。</summary>
        public static ImageType CSU_MLO                         = new ImageType(22, "CSU MLO", new byte[] { 0x10, 0x00, 0x00, 0x17 }, OpenDialogFilter.ALL);
        /// <summary>CCS board MLO。</summary>
        public static ImageType CCS_Board_MLO                   = new ImageType(23, "CCS board MLO", new byte[] { 0x10, 0x00, 0x00, 0x18 }, OpenDialogFilter.ALL);
        /// <summary>AC cord set controller。</summary>
        public static ImageType AC_CordSetController            = new ImageType(24, "AC cord set controller", new byte[] { 0x20, 0x00, 0x00, 0x01 }, OpenDialogFilter.HEX);
        /// <summary>AC wall-mount (low-end) controller。</summary>
        public static ImageType AC_WallMountController          = new ImageType(25, "AC wall-mount (low-end) controller", new byte[] { 0x20, 0x00, 0x00, 0x02 }, OpenDialogFilter.HEX);
        /// <summary>CMU in BMS。</summary>
        public static ImageType BMS_CMU                         = new ImageType(26, "CMU in BMS", new byte[] { 0x30, 0x00, 0x00, 0x01 }, OpenDialogFilter.HEX);
        /// <summary>BMU in BMS。</summary>
        public static ImageType BMS_BMU                         = new ImageType(27, "BMU in BMS", new byte[] { 0x30, 0x00, 0x00, 0x02 }, OpenDialogFilter.HEX);

        #region Properties
        /// <summary>取得對應使用者操作介面之索引。</summary>
        public ushort           Index { get; internal set; }
        /// <summary>取得文字描述。</summary>
        public string           Text { get; internal set; }
        /// <summary>取得型態代碼。</summary>
        public byte[]           CodeArray { get; internal set; }
        /// <summary>取得副檔名搜尋規則。</summary>
        public OpenDialogFilter Filter { get; internal set; }
        #endregion

        #region Constructor
        /// <summary>初始化 ImageType 類別的新執行個體。</summary>
        /// <param name="index"></param>
        /// <param name="text"></param>
        /// <param name="code"></param>
        /// <param name="filter"></param>
        public ImageType(ushort index, string text, byte[] code, OpenDialogFilter filter)
        {
            Index = index;
            Text = text;
            CodeArray = code;
            Filter = filter;
        }
        #endregion

        #region Methods
        /// <summary>取得所有裝置定義。</summary>
        /// <returns></returns>
        public static IEnumerable<ImageType> List
        {
            get
            {
                yield return CSU30_Bootloader;
                yield return CSU30_KernelConfig;
                yield return CSU30_KernelImage;
                yield return CSU30_RootFileSystem;
                yield return CSU30_UserConfig;
                yield return CSU30_PrimaryController;
                yield return CSU30_CCS_BoardBootloader;
                yield return CSU30_CCS_BoardKernelConfig;
                yield return CSU30_CCS_BoardKernelImage;
                yield return CSU30_CCS_BoardFileSystem;
                yield return CSU30_CAHdeMO_Board;
                yield return CSU30_GB_Board;
                yield return RelayControlBoard;
                yield return FanControlBoard;
                yield return LCM;
                yield return Infy_PFC_PrimaryController;
                yield return Infy_DD_SecondaryController;
                yield return F950_PSU_PrimaryController;
                yield return F950_PSU_SecondaryController;
                yield return LED_control_board;
                yield return LCM_APK;
                yield return LCM_Advertise_Video;
                yield return CSU_MLO;
                yield return CCS_Board_MLO;
                yield return AC_CordSetController;
                yield return AC_WallMountController;
                yield return BMS_CMU;
                yield return BMS_BMU;
            }
        }

        /// <summary>取得所有裝置定義。</summary>
        /// <returns></returns>
        public static IEnumerable<string> ListText
        {
            get
            {
                yield return CSU30_Bootloader.Text;
                yield return CSU30_KernelConfig.Text;
                yield return CSU30_KernelImage.Text;
                yield return CSU30_RootFileSystem.Text;
                yield return CSU30_UserConfig.Text;
                yield return CSU30_PrimaryController.Text;
                yield return CSU30_CCS_BoardBootloader.Text;
                yield return CSU30_CCS_BoardKernelConfig.Text;
                yield return CSU30_CCS_BoardKernelImage.Text;
                yield return CSU30_CCS_BoardFileSystem.Text;
                yield return CSU30_CAHdeMO_Board.Text;
                yield return CSU30_GB_Board.Text;
                yield return RelayControlBoard.Text;
                yield return FanControlBoard.Text;
                yield return LCM.Text;
                yield return Infy_PFC_PrimaryController.Text;
                yield return Infy_DD_SecondaryController.Text;
                yield return F950_PSU_PrimaryController.Text;
                yield return F950_PSU_SecondaryController.Text;
                yield return LED_control_board.Text;
                yield return LCM_APK.Text;
                yield return LCM_Advertise_Video.Text;
                yield return CSU_MLO.Text;
                yield return CCS_Board_MLO.Text;
                yield return AC_CordSetController.Text;
                yield return AC_WallMountController.Text;
                yield return BMS_CMU.Text;
                yield return BMS_BMU.Text;
            }
        }

        /// <summary>根據索引值，解析出對應的映像檔型態。</summary>
        /// <param name="index"></param>
        /// <returns></returns>
        public static ImageType Parse(ushort index)
        {
            foreach (ImageType type in List)
            {
                if (type.Index == index) {
                    return type;
                }
            }

            return null;
        }
        #endregion
    }
}
