﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace FirmwareHeaderBuilder.Model
{
    /// <summary>表示開啟檔案對話框過濾器之類別。</summary>
    public class OpenDialogFilter
    {
        public static OpenDialogFilter BIN = new OpenDialogFilter(0, "bin file (*.bin)|*.bin");

        public static OpenDialogFilter DTB = new OpenDialogFilter(1, "dtb file (*.dtb)|*.dtb");

        public static OpenDialogFilter GZ = new OpenDialogFilter(2, "gz file (*.gz)|*.gz");

        public static OpenDialogFilter HEX = new OpenDialogFilter(3, "hex file (*.hex)|*.hex");

        public static OpenDialogFilter IMAGE = new OpenDialogFilter(4, "image file (*.img)|*.img");

        public static OpenDialogFilter ALL = new OpenDialogFilter(5, "all file (*.*)|*.*");


        /// <summary>取得索引指標。</summary>
        public int      Index { get; set; }
        /// <summary>取得文字描述。</summary>
        public string   FilterText { get; set; }

        public OpenDialogFilter(int index, string filter)
        {
            Index = index;
            FilterText = filter;
        }

        /// <summary>取得所有濾器項目。</summary>
        public static IEnumerable<OpenDialogFilter> List
        {
            get
            {
                yield return BIN;
                yield return DTB;
                yield return GZ;
                yield return HEX;
                yield return IMAGE;
                yield return ALL;
            }
        }

        /// <summary>取得濾器項目規則文字。</summary>
        /// <returns></returns>
        public static string GetFilterRuleText()
        {
            OpenDialogFilter[] filters = List.ToArray();

            string text = string.Empty;
            for (int i = 0; i< filters.Length; i++)
            {
                if (i < filters.Length - 1)
                {
                    text += filters[i].FilterText + "|";
                }
                else
                {
                    text += filters[i].FilterText;
                }
            }

            return text;
        }

        /// <summary>根據索引指標，找到相對應的濾器項目。</summary>
        /// <param name="index"></param>
        /// <returns></returns>
        public static OpenDialogFilter Parse(int index)
        {
            foreach (OpenDialogFilter filter in List)
            {
                if (filter.Index == index)
                {
                    return filter;
                }
            }

            throw new Exception("OpenDialogFilter parse error");
        }

        /// <summary>根據文字描述，找到相對應的濾器項目。</summary>
        /// <param name="text"></param>
        /// <returns></returns>
        public static OpenDialogFilter Parse(string text)
        {
            foreach (OpenDialogFilter filter in List)
            {
                if (filter.FilterText == text)
                {
                    return filter;
                }
            }

            throw new Exception("OpenDialogFilter parse error");
        }
    }
}
