﻿using System;
using System.IO.Ports;
using System.Linq;
using System.Reflection;
using System.Security.Principal;
using System.Windows.Forms;
using Net.Core.Communication.Serial.Sync;
using PH.Bootloader.RS485;
using PH.Bootloader.RS485.Communication;

namespace IntegratedBootloader
{
    public partial class Form1 : Form
    {
        private bool                isCommReady;
        private ISyncCommunicationPlus syncComm;
        private DeviceType          deviceType;
        private DeviceInformation   targetDevice;
        private RS485Bootloader     bootloader;

        public Form1()
        {
            InitializeComponent();

            CheckForIllegalCrossThreadCalls = false;
            Text = Text + string.Format(" {0}{1}", "V", Assembly.GetEntryAssembly().GetName().Version);

            InitializeCustom();
            InitializeUI();
            ControlAllUI(false);
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            string strUserName = WindowsIdentity.GetCurrent().Name;
            string[] aryUserInfo = strUserName.Split(new string[] { "\\" }, StringSplitOptions.RemoveEmptyEntries);
            if (aryUserInfo.Length >= 2)
            {
                Billboard.AppendText(string.Format("Welcome~ {0}\r\n", aryUserInfo[1]));
            }
            else
            {
                Billboard.AppendText(string.Format("Welcome~ {0}\r\n", aryUserInfo[0]));
            }
        }

        private void InitializeCustom()
        {
            syncComm = new SerialPortHandler();
            syncComm.TxSilenceEnable = false;
            syncComm.StatusChanged += SyncComm_StatusChanged;

            bootloader = new RS485Bootloader();
            bootloader.MessageShown += Bootloader_MessageShown;
            bootloader.ProcedureProgressChanged += Bootloader_ProcedureProgressChanged;
            bootloader.UploadInformationChanged += Bootloader_UploadInformationChanged;
            bootloader.ProcedureCompleted += Bootloader_ProcedureCompleted;
        }

        private void InitializeUI()
        {
            ComOption.SelectedIndexChanged += ComOption_SelectedIndexChanged;
            ComOption.DropDown += ComOption_DropDown;
            ComOption.Items.Clear();
            ComOption.Items.Add("OFF");
            ComOption.Items.AddRange(SerialPort.GetPortNames());
            ComOption.SelectedIndex = 0;

            DeviceNameOption.SelectedIndexChanged += DeviceNameOption_SelectedIndexChanged;

            DeviceTypeOtion.SelectedIndexChanged += DeviceTypeOtion_SelectedIndexChanged;
            DeviceTypeOtion.Items.AddRange(DeviceType.ListDeviceName.ToArray());
            DeviceTypeOtion.SelectedIndex = 0;

            CheckBTN.Click += Button_Click;
            BrowserBTN.Click += Button_Click;
            UploadBTN.Click += Button_Click;
            AbortBTN.Click += Button_Click;
            AppCodeBTN.Click += Button_Click;
            HexFileText.DragEnter += HexFileText_DragEnter;
            HexFileText.DragDrop += HexFileText_DragDrop;
        }

        private void ControlAllUI(bool enable)
        {
            DeviceTypeOtion.Enabled = enable;
            DeviceNameOption.Enabled = enable;
            CheckBTN.Enabled = enable;
            BrowserBTN.Enabled = enable;
            UploadBTN.Enabled = enable;
            AbortBTN.Enabled = enable;
            AppCodeBTN.Enabled = enable;
            CanAgentChecker.Enabled = enable;
        }

        private void ControlIdleUI()
        {
            DeviceTypeOtion.Enabled = true;
            DeviceNameOption.Enabled = true;
            CheckBTN.Enabled = true;
            BrowserBTN.Enabled = true;
            UploadBTN.Enabled = false;
            AbortBTN.Enabled = false;
            AppCodeBTN.Enabled = true;
            CanAgentChecker.Enabled = true;
        }

        private void ControlImageReadyUI()
        {
            DeviceTypeOtion.Enabled = true;
            DeviceNameOption.Enabled = true;
            CheckBTN.Enabled = true;
            BrowserBTN.Enabled = true;
            UploadBTN.Enabled = true;
            AbortBTN.Enabled = false;
            AppCodeBTN.Enabled = true;
            CanAgentChecker.Enabled = true;
        }

        private void ControlRunningUI()
        {
            DeviceTypeOtion.Enabled = false;
            DeviceNameOption.Enabled = false;
            CheckBTN.Enabled = false;
            BrowserBTN.Enabled = false;
            UploadBTN.Enabled = false;
            AbortBTN.Enabled = true;
            AppCodeBTN.Enabled = false;
            CanAgentChecker.Enabled = false;
        }
    }
}
