﻿using System;
using System.Drawing;
using System.IO;
using System.IO.Ports;
using System.Linq;
using System.Reflection;
using System.Threading;
using System.Windows.Forms;
using Microsoft.VisualBasic;
using Net.Core.Communication.Serial.Sync;
using PH.Bootloader.RS485.Communication;
using PH.Bootloader.RS485.PIC32_2;
using PH.Bootloader.RS485.PIC32_2.Protocol;
using PIC32Bootloader = PH.Bootloader.RS485.PIC32_2.PIC32Bootloader2;

namespace PIC32Bootloader2
{
    public partial class Form1 : Form
    {
        private const int           COMMUNICATION_SYNC_TIME = 2000;

        private SerialPortHandler   serialPortHandler;
        private PIC32Bootloader     pic32Bootloader;
        private DeviceInformation   targetDevice;
        private UpgradeImageType    upgradeType;

        /// <summary>
        /// 
        /// </summary>
        public Form1()
        {
            InitializeComponent();
            InitializeCustom();
        }

        private void InitializeCustom()
        {
            CheckForIllegalCrossThreadCalls = false;

            Text = Text + string.Format(" V{0}", Assembly.GetEntryAssembly().GetName().Version);

            serialPortHandler = new SerialPortHandler();
            serialPortHandler.SetSyncTime(COMMUNICATION_SYNC_TIME);
            serialPortHandler.StatusChanged += SerialPortHandler_StatusChanged;

            pic32Bootloader = new PIC32Bootloader();
            pic32Bootloader.SetCommSyncTime(COMMUNICATION_SYNC_TIME);
            pic32Bootloader.UpgradeStatusChanged += Pic32Bootloader_UpgradeStatusChanged;
            pic32Bootloader.UpgradeProgressChanged += Pic32Bootloader_UpgradeProgressChanged;

            optComm.SelectedIndexChanged += OptionComm_SelectedIndexChanged;
            optComm.Items.Clear();
            optComm.Items.Add("OFF");
            optComm.Items.AddRange(SerialPort.GetPortNames());

            optDeviceType.SelectedIndexChanged += OptDeviceType_SelectedIndexChanged;
            optDeviceType.Items.Clear();
            optDeviceType.Items.AddRange(DeviceType.ListDeviceName.ToList().ToArray());

            optDeviceID.SelectedIndexChanged += OptDeviceID_SelectedIndexChanged;

            btnCheck.Click += BtnCheck_Click;
            btnBrowse.Click += BtnBrowse_Click;
            btnUpgrade.Click += BtnUpgrade_Click;
            btnAbort.Click += BtnAbort_Click;

            txtBillboard.DragEnter += TxtBillboard_DragEnter;
            txtBillboard.DragDrop += TxtBillboard_DragDrop;

            optUpgradeType.Items.Clear();
            optUpgradeType.Items.Add("AppCode");
            optUpgradeType.Items.Add("AppCode (By BootCode)");
            optUpgradeType.Items.Add("BootCode");
            optUpgradeType.SelectedIndexChanged += OptUpgradeType_SelectedIndexChanged;

            optComm.SelectedIndex = 0;
            optDeviceType.SelectedIndex = 0;
            optUpgradeType.SelectedIndex = 0;
        }

        private void OptUpgradeType_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (optUpgradeType.SelectedIndex == 0)
            {
                upgradeType = UpgradeImageType.AppCode;
            }
            if (optUpgradeType.SelectedIndex == 1)
            {
                upgradeType = UpgradeImageType.AppCodeByBootCode;
            }
            else if (optUpgradeType.SelectedIndex == 2)
            {
                string input = Interaction.InputBox("Enter Password", "System Security", "", -1, -1);
                if (input == "741500")
                {
                    upgradeType = UpgradeImageType.BootCode;
                }
                else
                {
                    optUpgradeType.SelectedIndex = 0;
                }
            }

            pic32Bootloader.Clear();
        }

        private void SerialPortHandler_StatusChanged(SyncCommStatus status)
        {
            switch (status)
            {
                case SyncCommStatus.Closed:
                        PostBillboard("通訊埠已關閉");
                        panelComm.BackColor = serialPortHandler.IsOpen ? Color.Green : Color.Silver;
                        break;
                case SyncCommStatus.InUse:
                        PostBillboard("開啟通訊埠失敗，通訊埠已被開啟"); break;
                case SyncCommStatus.NotExist:
                        PostBillboard("開啟通訊埠失敗，找不到通訊埠"); break;
                case SyncCommStatus.Opened:
                        PostBillboard("通訊埠已開啟");
                        panelComm.BackColor = serialPortHandler.IsOpen ? Color.Green : Color.Silver;
                        break;
                case SyncCommStatus.WrongName:
                        PostBillboard("開啟通訊埠失敗，錯誤的通訊埠名稱"); break;
                case SyncCommStatus.Others:
                default:
                        PostBillboard("未定義的錯誤!"); break;
            }
        }

        private void Pic32Bootloader_UpgradeStatusChanged(object sender, UpgradeStatusType errorType)
        {
            switch (errorType)
            {
                case UpgradeStatusType.FlashErasing:
                        PostBillboard("抹除記憶體中....."); break;
                case UpgradeStatusType.EraseFlashCommandFailed:
                        PostBillboard("抹除記憶體命令執行失敗"); break;
                case UpgradeStatusType.EraseFlashFailed:
                        PostBillboard("抹除記憶體失敗"); break;
                case UpgradeStatusType.EraseFlashFinished:
                        PostBillboard("抹除記憶體完成"); break;
                case UpgradeStatusType.EraseFlashSizeError:
                        PostBillboard("抹除記憶體大小錯誤"); break;
                case UpgradeStatusType.HasOthersProcessRunning:
                        PostBillboard("有其他程序正在執行中"); break;
                case UpgradeStatusType.ImageFileNotAppCode:
                        PostBillboard("映像檔非應用程式碼"); break;
                case UpgradeStatusType.ImageFileNotExist:
                        PostBillboard("映像檔不存在"); break;
                case UpgradeStatusType.ImageFileNotMatch:
                        PostBillboard("映像檔與目標不匹配"); break;
                case UpgradeStatusType.ImageFileReady:
                        PostBillboard("載入、解析映像檔完成"); break;
                case UpgradeStatusType.ImageFileEmpty:
                        PostBillboard("映像檔無內容"); break;
                case UpgradeStatusType.MemoryAllocationFailed:
                        PostBillboard("記憶體配置失敗"); break;
                case UpgradeStatusType.MemoryCopying:
                        PostBillboard("記憶體複製中....."); break;
                case UpgradeStatusType.MemoryCopyCommandFailed:
                        PostBillboard("記憶體複製命令執行失敗"); break;
                case UpgradeStatusType.MemoryCopyEmptyPage:
                        PostBillboard("記憶體複製無回應或頁數為0"); break;
                case UpgradeStatusType.MemoryCopyNotDone:
                        PostBillboard("記憶體複製沒收到結束命令"); break;
                case UpgradeStatusType.MemoryCopyFinished:
                        PostBillboard("記憶體複製完成"); break;
                case UpgradeStatusType.MemoryCopyNoVerified:
                        PostBillboard("記憶體複製後沒有接收到確認命令"); break;
                case UpgradeStatusType.MemoryCopySizeErorr:
                        PostBillboard("記憶體複製資料大小錯誤"); break;
                case UpgradeStatusType.MemoryInvalidCRC:
                        PostBillboard("記憶體內容 CRC 驗證失敗"); break;
                case UpgradeStatusType.MemoryNotAllocated:
                        PostBillboard("記憶體內容尚未配置，請載入映像檔"); break;
                case UpgradeStatusType.SerialPortNotReady:
                        PostBillboard("通訊埠尚未備妥"); break;
                case UpgradeStatusType.TriggerFinished:
                        PostBillboard("觸發裝置完成"); break;
                case UpgradeStatusType.TriggerInvalidResponse:
                        PostBillboard("觸發裝置無效的回應"); break;
                case UpgradeStatusType.TriggerNoResponse:
                        PostBillboard("觸發裝置無回應"); break;
                case UpgradeStatusType.TriggerFailed:
                        PostBillboard("觸發裝置失敗5次"); break;
                case UpgradeStatusType.Upgrading:
                        PostBillboard("程式碼資料更新中....."); break;
                case UpgradeStatusType.UpgradeInvalidAddress:
                        PostBillboard("程式碼更新不合法的位址"); break;
                case UpgradeStatusType.UpgradeNoResponse:
                        PostBillboard("程式碼更新時無法得到回應"); break;
                case UpgradeStatusType.UpgradeResponseUndefine:
                        PostBillboard("程式碼更新時得到未定義的回應"); break;
                case UpgradeStatusType.UpgradeFinished:
                        PostBillboard("程式碼更新完成 ★★★★★");
                        PostBillboard("應用程式執行中.....");
                        PostBillboard(""); break;
                case UpgradeStatusType.UpgradeFailed:
                        PostBillboard("程式碼更新失敗"); break;
                case UpgradeStatusType.VerifyCommandFailed:
                        PostBillboard("程式碼確效指令失敗"); break;
                case UpgradeStatusType.VerifyFinished:
                        PostBillboard("程式碼確效完成"); break;
                case UpgradeStatusType.VerifyNoResponse:
                        PostBillboard("程式碼確效無回應"); break;
                default:
                        PostBillboard("未定義的錯誤訊息"); break;
            }
        }

        private void Pic32Bootloader_UpgradeProgressChanged(object sender, UpgradeProgressType progressType, int percentage)
        {
            string txt = string.Empty;
            switch (progressType)
            {
                case UpgradeProgressType.CopyFlash:
                    txt = percentage == 0 ? string.Empty : string.Format("記憶體複製進度 {0}%", percentage); break;
                case UpgradeProgressType.EraseFlash:
                    txt = percentage == 0 ? string.Empty : string.Format("記憶體抹除進度 {0}%", percentage); break;
                case UpgradeProgressType.Upgrade:
                    txt = percentage == 0 ? string.Empty : string.Format("程式碼更新進度 {0}%", percentage); break;
            }
            labProcedure.Text = txt;
            progessProcedure.Value = percentage;
        }

        private void OptionComm_SelectedIndexChanged(object sender, System.EventArgs e)
        {
            string name = (string)optComm.SelectedItem;
            if (name.Equals("OFF"))
            {
                serialPortHandler.Close();
            }
            else
            {
                serialPortHandler.Open(name, 115200);
                pic32Bootloader.AssignComPort(serialPortHandler);
            }
        }

        private void OptDeviceType_SelectedIndexChanged(object sender, System.EventArgs e)
        {
            DeviceType deviceType = DeviceType.ParseByIndex(optDeviceType.SelectedIndex);
            optDeviceID.Items.Clear();
            optDeviceID.Items.AddRange(DeviceInformation.SpecifiedDeviceNames(deviceType));
            optDeviceID.SelectedIndex = 0;
        }

        private void OptDeviceID_SelectedIndexChanged(object sender, System.EventArgs e)
        {
            targetDevice = DeviceInformation.ParseByName((string)optDeviceID.SelectedItem);
        }

        private void BtnCheck_Click(object sender, EventArgs e)
        {
            if (pic32Bootloader.CheckCommlPort())
            {
                string version = pic32Bootloader.GetVersion(targetDevice);
                PostBillboard(string.IsNullOrEmpty(version) ? "找不到裝置或裝置中沒有預載啟動程序" : version);
            }
        }

        private void BtnBrowse_Click(object sender, EventArgs e)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.Title = "Select HEX";
            dialog.InitialDirectory = ".\\";
            dialog.Filter = "hex files (*.*)|*.hex";

            if (dialog.ShowDialog() == DialogResult.OK) {

                ClearBillboard();
                PostBillboard(dialog.FileName);
                pic32Bootloader.LoadHexFile(targetDevice, dialog.FileName, upgradeType);
            }   
        }

        private void BtnUpgrade_Click(object sender, System.EventArgs e)
        {
            if (pic32Bootloader.CheckCommlPort())
            {
                bool isSuccess = false;

                if (upgradeType == UpgradeImageType.AppCodeByBootCode)
                {
                    isSuccess = pic32Bootloader.Reset(targetDevice, upgradeType);
                    if (isSuccess) {
                        Thread.Sleep(200);
                        ClearBillboard();
                        pic32Bootloader.Upgrade(upgradeType);
                    }
                }
                else
                {
                    isSuccess = pic32Bootloader.UpgradeEnable(targetDevice, upgradeType);
                    // 當只有 Boot Code 時，更新致能的指令不會得到回覆
                    if (!isSuccess) {
                        if (MessageBox.Show("程式碼更新致能命令未得到回應!!\r\n是否繼續執行更新?", "程式碼更新", MessageBoxButtons.YesNo, MessageBoxIcon.Information) == DialogResult.Yes) {
                            Thread.Sleep(200);
                            ClearBillboard();
                            pic32Bootloader.Upgrade(upgradeType);
                        }
                    }
                    else
                    {
                        Thread.Sleep(200);
                        ClearBillboard();
                        pic32Bootloader.Upgrade(upgradeType);
                    }
                } 
            }    
        }

        private void BtnAbort_Click(object sender, System.EventArgs e)
        {
            pic32Bootloader.Abort();
        }

        private void TxtBillboard_DragEnter(object sender, DragEventArgs e)
        {
            e.Effect = e.Data.GetDataPresent(DataFormats.FileDrop) ?
                       DragDropEffects.All :
                       DragDropEffects.None;
        }

        private void TxtBillboard_DragDrop(object sender, DragEventArgs e)
        {
            string fileName = ((string[])e.Data.GetData(DataFormats.FileDrop)).GetValue(0).ToString();

            if (string.Equals(Path.GetExtension(fileName), ".hex"))
            {
                if (!pic32Bootloader.IsProcedureRunning)
                {
                    PostBillboard(fileName);
                    pic32Bootloader.LoadHexFile(targetDevice, fileName, upgradeType);
                }
                else
                {
                    PostBillboard("更新程序進行中，請勿重複載入檔案");
                }
            }
            else
            {
                PostBillboard("載入非映像檔檔案");
            }
        }

        private void ClearBillboard()
        {
            txtBillboard.Clear();
        }

        private void PostBillboard(string message)
        {
            txtBillboard.AppendText(message + Environment.NewLine);
        }
    }
}
