﻿using PH.Bootloader.RS485;
using System;
using System.IO.Ports;
using System.Windows.Forms;
using Net.Core.Communication.Serial.Sync;
using PH.Bootloader.RS485.Communication;

namespace UnitTest_Bootloader.RS485
{
    public partial class Form1 : Form
    {
        private ISyncCommunicationPlus  serialPort;
        private DeviceType              deviceType;
        private DeviceInformation       targetDevice;
        private RS485Bootloader         bootloader;

        public Form1()
        {
            InitializeComponent();

            CheckForIllegalCrossThreadCalls = false;

            serialPort = new SerialPortHandler();
            serialPort.StatusChanged += SerialPort_StatusChanged; ;

            bootloader = new RS485Bootloader();
            bootloader.MessageShown += Bootloader_MessageShown;
            bootloader.ProcedureProgressChanged += Bootloader_ProcedureProgressChanged;
            bootloader.ProcedureCompleted += Bootloader_ProcedureCompleted;  
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            CommOption.Items.Clear();
            CommOption.Items.Add("OFF");
            CommOption.Items.AddRange(SerialPort.GetPortNames());
            CommOption.SelectedIndex = 0;

            radioButton1.CheckedChanged += RadioButton_CheckedChanged;
            radioButton2.CheckedChanged += RadioButton_CheckedChanged;
            radioButton3.CheckedChanged += RadioButton_CheckedChanged;
            radioButton1.Checked = true;
        }

        private void CommOption_SelectedIndexChanged(object sender, EventArgs e)
        {
            string commName = (string)CommOption.SelectedItem;

            if (commName.Equals("OFF")) {
                serialPort.Close();
            } else {
                serialPort.Open(commName, 115200);
            }
        }

        private void GetBootloaderBTN_Click(object sender, EventArgs e)
        {
            bootloader.SetBootloaderType(deviceType);
        }

        private void CheckDeviceBTN_Click(object sender, EventArgs e)
        {
            bootloader.CheckDevice(targetDevice);
        }

        private void BrowserHexBTN_Click(object sender, EventArgs e)
        {
            OpenFileDialog openFileDialog = new OpenFileDialog();
            openFileDialog.Filter = "hex files (*.hex*)|*.hex*|All files (*.*)|*.*";
            openFileDialog.FilterIndex = 1;
            openFileDialog.RestoreDirectory = true;

            if (openFileDialog.ShowDialog() == DialogResult.OK) {
                bootloader.ParseHexToMemory(targetDevice, openFileDialog.FileName);
            }
        }

        private void UploadBTN_Click(object sender, EventArgs e)
        {
            bootloader.Upload(targetDevice);
        }

        private void AbortBTN_Click(object sender, EventArgs e)
        {
            bootloader.Abort();
        }

        private void CheckAppBTN_Click(object sender, EventArgs e)
        {
            bool isOK = bootloader.CheckAppCode(targetDevice);
            MessageBox.Show(isOK ? "成功" : "失敗");
        }

        private void RadioButton_CheckedChanged(object sender, EventArgs e)
        {
            RadioButton radioButton = sender as RadioButton;

            deviceType = radioButton.Name.Equals("radioButton1") ? DeviceType.PIC_CSU :
                         radioButton.Name.Equals("radioButton2") ? DeviceType.PIC_PSU : DeviceType.DSP_PSU;

            DeviceOption.Items.Clear();
            DeviceOption.Items.AddRange(DeviceInformation.GetNameList(deviceType));
            DeviceOption.SelectedIndex = 0;
        }

        private void DeviceOption_SelectedIndexChanged(object sender, EventArgs e)
        {
            string name = (string)DeviceOption.SelectedItem;
            targetDevice = DeviceInformation.ParseByName(name);
        }

        private void SerialPort_StatusChanged(SyncCommStatus status)
        {
            string message = string.Empty;

            if (status == SyncCommStatus.Opened)
            {
                message = "通訊埠開啟";
                bootloader.AssignComPort(serialPort);
            }
            else if (status == SyncCommStatus.Closed)
            {
                message = "通訊埠關閉";
            }
            else
            {
                message = "通訊埠發生錯誤";
            }

            listBox1.Items.Add(message);
        }

        private void Bootloader_ProcedureCompleted(object sender, ProcedureType procType, BootloadResult result)
        {
            if (procType == ProcedureType.CheckCommunication)
            {

            }
            else if (procType == ProcedureType.CheckDevice)
            {

            }
            else if (procType == ProcedureType.HexToMemory)
            {

            }
            else
            {

            }
        }

        private void Bootloader_ProcedureProgressChanged(object sender, ProcedureType procType, int progressPercentage)
        {
            progressBar1.Value = progressPercentage;
        }

        private void Bootloader_MessageShown(object sender, string message)
        {
            listBox1.Items.Add(message);
            listBox1.SelectedIndex = listBox1.Items.Count - 1;
        }

        
    }
}
